# $Id: Request.pm 51836 2024-12-02 11:25:08Z wsl $
# $URL: https://svn.uvt.nl/its-id/trunk/sources/kiki/lib/UvT/Kiki/WS/Person/Post/Request.pm $

use utf8;
use strict;
use warnings FATAL => 'all';

package UvT::Kiki::WS::Person::Post::Request;

use JSON;

use UvT::Kiki::WS::Person::Request -self;

field notes => {name => '', domain => '', destinations => []};

field request_data => sub {
	my $self = shift;
	my $type = $self->content_type;
	die "Unsupported request content type '$type' (expected application/json)\n"
		if $type ne 'application/json';
	return decode_json($self->request_body->bodyhandle->as_string);
};

field new_person => sub {
	my $self = shift;
	my $anr = $self->anr;
	my $person = super || $self->db->new_person($anr);
	my $notes = $self->notes;

	if($self->nonce) {
		my $db = $self->db;
		my $data = $self->request_data;
		my $is_robot = $self->is_robot;

		if($person->type eq 'alias') {
			$notes->{update} = [error => 'alias'];
			return $person;
		}

		unless($person->original) {
			if($anr < 100000 || $anr > 999999) {
				$notes->{update} = [error => 'invalid-anr'];
				return $person;
			}
			unless(exists $data->{labels} && exists $data->{mailbox}) {
				$notes->{update} = [error => 'incomplete'];
				return $person;
			}
			unless($is_robot) {
				$notes->{update} = [error => 'no-robot'];
				return $person;
			}
		}

		my @all_notes;
		if(exists $data->{labels}) {
			my @labels = map { trim($_) } @{$data->{labels}};
			my $label_notes = $person->update_labels(\@labels, !$is_robot);
			$notes->{labels} = $label_notes;
			push @all_notes, @$label_notes;
		}
		if(exists $data->{mailbox}) {
			my $mailbox_note = $person->update_mailbox(trim($data->{mailbox}));
			$notes->{mailbox} = $mailbox_note;
			push @all_notes, $mailbox_note;
		}

		if(grep { $_->[0] eq 'error' } @all_notes) {
			$notes->{update} = ['error'];
		} else {
			my $what = $is_robot ? "Robot" : "User";
			my $who = $self->login;
			my $new = $person->toString;
			my $full = $person->full;
			my $anr = sprintf('%06d', $anr);
			if(my $original = $person->original) {
				my $old = $original->toString;
				warn "$what $who updated person $anr:\n\told: $old\n\tnew: $new\n";
				$self->mailtje("[kiki] $who updated $full ($anr)", $person->diff);
			} else {
				warn "$what $who created person $anr:\n\tnew: $new\n";
				$self->mailtje("[kiki] $who created $full ($anr)", $person->diff);
			}

			$person->store;
			$self->activate;
			$notes->{update} = [ok => 'stored'];
		}
	} else {
		$notes->{update} = [error => 'session'];
	}

	return $person;
};
